#!/bin/bash

PHARMGKB=""
HLASCAN=""
BAM=""
VCF=""
OUTPUTDIR=""
PRINT_HELP=0

while [[ "$#" -gt 0 ]]; do
    case $1 in
        -p|--pharmgkb) PHARMGKB="$2"; shift ;;
        -s|--hlascan) HLASCAN="$2"; shift ;;
        -b|--bam) BAM="$2"; shift ;;
        -v|--vcf) VCF="$2" ; shift ;;
	-o|--out-directory) OUTPUTDIR="$2" ; shift ;;
	-h|--help) PRINT_HELP=1 ;;
        *) echo "Unknown parameter passed: $1"; exit 0 ;;
    esac
    shift
done

if [ ${PRINT_HELP} -eq 1 ]
then
	echo "Usage:"
	echo ""
	echo "bash Extract_PharmGKB_Variants.sh -p <path/to/PharmGKB/file> -b <path/to/bam/file> -s <path/to/hlascan/files> -v <path/to/vcf/file>  -o <path/to/output/dir> -h [print/this/help]"
	echo ""
	echo "<Mandatory>: "
	echo "-p <PharmGKB file>: Path to PharmGKB tsv file download at PharmGKB site."
	echo "-b <bam file>: Bam file provided by alignement."
	echo "-s <path to hlascan files>: Path to HLAscan tools download and unzipped."
	echo "-v <vcf file>: vcf file provided by variant calling"
	echo "-o <OUTPUT_PATH>: Output path."
	echo ""
	exit 1
fi

# Check pharmgkb file is provided and exists
if [ "${PHARMGKB}" == "" ]
then
	1>&2 echo "Missing input pharmgkb tsv file (-p). Abort."
	exit 2
elif [ ! -f "${PHARMGKB}" ]
then
	1>&2 echo "pharmgkb tsv file not found. Abort."
	exit 3
fi

# Check  BAM file is provided and exists
if [ "${BAM}" == "" ]
then
	1>&2 echo "Missing input BAMs (-b). Abort."
	exit 4
elif [ ! -f "${BAM}" ]
then
	1>&2 echo "BAM file not found. Abort."
	exit 5
fi


# Check hlascan dir is provided and exists
if [ "${HLASCAN}" == "" ]
then
	1>&2 echo "Missing input HLAscan path (-s). Abort."
	exit 6
elif [ ! -d "${HLASCAN}" ]
then
	1>&2 echo "HLAscan path not found. Abort."
	exit 7
fi

# Check vcf file is provided and exists
if [ "${VCF}" == "" ]
then
	1>&2 echo "Missing input vcf (-v). Abort."
	exit 8
elif [ ! -f "${VCF}" ]
then
	1>&2 echo "vcf file not found. Abort."
	exit 9
fi

#  Check output dir is provided and exists
if [ "${OUTPUTDIR}" == "" ]
then
	1>&2 echo "Missing input output dir path (-s). Abort."
	exit 10
elif [ ! -d "${OUTPUTDIR}" ]
then
	1>&2 echo "Output dir path not found. Abort."
	exit 11
fi

# Check python version and PATH 
pythonversion=$(python2.7 -V 2>&1)
PythonExitCode=$?

if [ $PythonExitCode -eq 0 ]
then
	echo "python2.7 is in PATH, continue"
else
	echo "missing python2.7 in PATH, use alias if necessary"
	exit 12
fi

version=$(echo $pythonversion | cut -f2 -d" " | cut -f1,2 -d.)
if [ $version != '2.7' ]
then
	echo "not the python version supported, please use python2.7"
	exit 13
fi

# find pipeline directory
PIPELINEPATH=$(dirname $0)

echo "Input pharmgkb file: $PHARMGKB"
echo "Path to HLAscan: $HLASCAN"
echo "Path to create_html.py: $PIPELINEPATH/create_html.py"
echo "HLAscan executable: $HLASCAN/hla-paper/haplo_scan_v4.0-hla.py"
echo "Gene list for HLAscan: $HLASCAN/hla-ref-5gene/gene_list"
echo "Input bam file: $BAM"
echo "Input vcf file: $VCF"
echo "Output directory: ${OUTPUTDIR}"
echo "Output HLAscan: ${OUTPUTDIR}/HLAscan_result/"
echo "Output HTML file: $OUTPUTDIR/Report.html" 

#HLASCAN
python2.7 $HLASCAN/hla-paper/haplo_scan_v4.0-hla.py $BAM $HLASCAN/hla-ref-5gene/gene_list $OUTPUTDIR/HLAscan_result/ 1>> $OUTPUTDIR/HLAscan_log.txt 2>> $OUTPUTDIR/HLAscan_log.txt

#create HTML
python2.7 $PIPELINEPATH/create_html.py -v $VCF -s $OUTPUTDIR/HLAscan_result/Report -t $PHARMGKB -o $OUTPUTDIR/Report.html