### PharmAnnot PIPELINE ###
## create_html.py
## Version : 1.0.0
## Description : Create a html file with vcf file, BAM file and report from HLAscan
## Usage : python create_html.py -v vcf_SNP -s report_HLA -b BAM -t tsvfile -o html -e log
## Output : Annotated vcf file
## Requirements : python 2.7+ , vcf , htmltag
## Author : Simon.Verdez@u-bourgogne.fr
## Known bugs : None

import sys
import getopt
import subprocess
import csv
from pysam import VariantFile
import htmltag
import logging
from operator import *

def parsePharmGKB(tsvfile):
    """ Parse PharmGKB file """
    DictRs = dict()
    DictHLA = dict()
    with open(tsvfile,'r') as PharmGKB:
        reader = csv.DictReader(PharmGKB, delimiter='\t')
        for row in reader:
            if row['Variant/Haplotypes'].startswith('rs'):
                variant = row['Variant/Haplotypes']
                gene = row['Gene']
                LevelEvi = row['Level of Evidence']
                LevelModi = row['Level Modifiers']
                Drug = row['Drug(s)']
                Pheno = row['Phenotype(s)']
                Publi = row['URL']
                DictRs.setdefault(variant,{})
                DictRs[variant].setdefault(Drug,{})
                DictRs[variant][Drug].setdefault(Pheno,{})
                DictRs[variant][Drug][Pheno]["LevelEvi"] = LevelEvi
                DictRs[variant][Drug][Pheno]["LevelMod"] = LevelModi
                DictRs[variant][Drug][Pheno]["Publi"] = Publi
                if gene == "":
                    DictRs[variant][Drug][Pheno]["Gene"] = "intron"
                else:
                    DictRs[variant][Drug][Pheno]["Gene"] = gene
                
            elif row['Variant/Haplotypes'].startswith('HLA'):
                allele = row['Variant/Haplotypes'].split(":")[0]+":"+row['Variant/Haplotypes'].split(":")[1]
                gene = row['Gene']
                LevelEvi = row['Level of Evidence']
                LevelModi = row['Level Modifiers']
                Drug = row['Drug(s)']
                Pheno = row['Phenotype(s)']
                Publi = row['URL']
                DictHLA.setdefault(allele,{})
                DictHLA[allele].setdefault(Drug,{})
                DictHLA[allele][Drug].setdefault(Pheno,{})
                DictHLA[allele][Drug][Pheno]["LevelEvi"] = LevelEvi
                DictHLA[allele][Drug][Pheno]["LevelMod"]= LevelModi
                DictHLA[allele][Drug][Pheno]["Publi"] = Publi
                if gene == "":
                    DictHLA[allele][Drug][Pheno]["Gene"] = "intron"
                else:
                    DictHLA[allele][Drug][Pheno]["Gene"] = gene

    return DictRs,DictHLA

def contructHeader():
    """ Header construction """
    head = htmltag.HEAD()
    head = htmltag.append(htmltag.TITLE('pharmacogenetic'))
    head = head.append(htmltag.STYLE(type = "text/css").append("body { padding-left: 15em; font-family: Georgia; color:black }"))
    head = head.append(htmltag.STYLE(type = "text/css").append("h2 { padding-left: 15em;font-family: Georgia;color:blue }"))
    head = head.append(htmltag.STYLE(type = "text/css").append("ul.navbar { list-style-type: none; padding: 0; margin: 0; position: absolute; top: 2em; left: 1em; width: 13em }"))
    head = head.append(htmltag.STYLE(type = "text/css").append("ul.navbar li { background: yellow; margin: 0.5em 0; padding: 1em; border-left: 0.5em solid black }"))
    head = head.append(htmltag.STYLE(type = "text/css").append("P1 { color: red }"))
    head = head.append(htmltag.STYLE(type = "text/css").append("P2 { color: orange }"))

    body = htmltag.BODY()
    body = body.append(htmltag.UL(CLASS = "navbar").append(htmltag.UL(htmltag.LI(htmltag.A(href = "https://www.pharmgkb.org").append("pharmgkb")))).append(htmltag.UL(htmltag.LI(htmltag.A(href = "https://cpicpgx.org").append("cpicpgx")))))
    body = body.append(htmltag.H2('Variants pharmgkb'))
    
    return head,body


def writeBodyRS(Pheno,Drug,gene,LevelEvi,Publi,LevelModi,variant,body):
    """to facilitate rs extraction """
    if LevelEvi == "1A" or LevelEvi == "1B":
        body = body.append(htmltag.P(htmltag.P1("Phenotype: "+Pheno))+htmltag.P(htmltag.P1("Drug: "+Drug))+htmltag.P(htmltag.P1("gene: "+gene))+htmltag.P(htmltag.P1("variant: "+variant))+htmltag.P(htmltag.P1("Level of Evidence: "+LevelEvi))+htmltag.P(htmltag.P1("Publication: "+Publi))+htmltag.P(htmltag.P1("Level Modifier: "+LevelModi)))
    elif LevelEvi == "2A" or LevelEvi == "2B":
        body = body.append(htmltag.P(htmltag.P2("Phenotype: "+Pheno))+htmltag.P(htmltag.P2("Drug: "+Drug))+htmltag.P(htmltag.P2("gene: "+gene))+htmltag.P(htmltag.P2("variant: "+variant))+htmltag.P(htmltag.P2("Level of Evidence: "+LevelEvi))+htmltag.P(htmltag.P2("Publication: "+Publi))+htmltag.P(htmltag.P2("Level Modifier: "+LevelModi)))
    else :
        body = body.append(htmltag.P(htmltag.P("Phenotype: "+Pheno))+htmltag.P(htmltag.P("Drug: "+Drug))+htmltag.P(htmltag.P("gene: "+gene))+htmltag.P(htmltag.P("variant: "+variant))+htmltag.P(htmltag.P("Level of Evidence: "+LevelEvi))+htmltag.P(htmltag.P("Publication: "+Publi))+htmltag.P(htmltag.P("Level Modifier: "+LevelModi)))
    body = body.append(htmltag.hr())
    
    return body

def extractRS(data,vcfSNP,DictRs,body):
    """ rs information extraction """
    countSNV=0
    vcf_reader_SNP = VariantFile(vcfSNP)
    for record in vcf_reader_SNP.fetch():
        if record.id in DictRs:
            countSNV+=1
            for Drug in DictRs[record.id]:
                for Pheno in DictRs[record.id][Drug]:
                    gene = DictRs[record.id][Drug][Pheno]["Gene"]
                    LevelEvi = DictRs[record.id][Drug][Pheno]["LevelEvi"]
                    LevelModi = DictRs[record.id][Drug][Pheno]["LevelMod"]
                    Publi = DictRs[record.id][Drug][Pheno]["Publi"]
                    body =writeBodyRS(Pheno,Drug,gene,LevelEvi,Publi,LevelModi,record.id,body)

    body = body.append(htmltag.H3("Number of SNV found = "+str(countSNV)))
    return body


def writeBodyHLA(HLA,DictHLA,body):
    """to facilitate HLA extraction """
    if HLA in DictHLA:
        for Drug in DictHLA[HLA]:
            for Pheno in DictHLA[HLA][Drug]:
                gene = DictHLA[HLA][Drug][Pheno]["Gene"]
                LevelEvi = DictHLA[HLA][Drug][Pheno]["LevelEvi"]
                LevelModi = DictHLA[HLA][Drug][Pheno]["LevelMod"]
                Publi = DictHLA[HLA][Drug][Pheno]["Publi"]
                body = body.append(htmltag.hr())
                if LevelEvi == "1A" or LevelEvi == "1B":
                    body = body.append(htmltag.P(htmltag.P1("Phenotype: "+Pheno))+htmltag.P(htmltag.P1("Drug: "+Drug))+htmltag.P(htmltag.P1("allele: "+HLA))+htmltag.P(htmltag.P1("Level of Evidence: "+LevelEvi))+htmltag.P(htmltag.P1("Publication: "+Publi))+htmltag.P(htmltag.P1("Level Modifier: "+LevelModi)))
                elif LevelEvi == "2A" or LevelEvi == "2B":
                    body = body.append(htmltag.P(htmltag.P2("Phenotype: "+Pheno))+htmltag.P(htmltag.P2("Drug: "+Drug))+htmltag.P(htmltag.P2("allele: "+HLA))+htmltag.P(htmltag.P2("Level of Evidence: "+LevelEvi))+htmltag.P(htmltag.P2("Publication: "+Publi))+htmltag.P(htmltag.P2("Level Modifier: "+LevelModi)))
                else :
                    body = body.append(htmltag.P(htmltag.P("Phenotype: "+Pheno))+htmltag.P(htmltag.P("Drug: "+Drug))+htmltag.P(htmltag.P("allele: "+HLA))+htmltag.P(htmltag.P("Level of Evidence: "+LevelEvi))+htmltag.P(htmltag.P("Publication: "+Publi))+htmltag.P(htmltag.P("Level Modifier: "+LevelModi)))
                
    return body

def extractHLA(report,DictHLA,body):
    """ HLA information extraction """
    with open(report,'r') as Report_HLA:
        HLA = Report_HLA.readlines()
        ListeA = []
        ListeB = []
        ListeC = []
        ListeDQB1 = []
        ListeDRB1 = []
        for lines in HLA :
            if lines.startswith("A"):
                for ligne in lines.split("\t"):
                    if ligne.startswith("A*"):
                        ListeA.append(ligne)
            if lines.startswith("B"):
                for ligne in lines.split("\t"):
                    if ligne.startswith("B*"):
                        ListeB.append(ligne)
            if lines.startswith("C"):
                for ligne in lines.split("\t"):
                    if ligne.startswith("C*"):
                        ListeC.append(ligne)
            if lines.startswith("DQB1"):
                for ligne in lines.split("\t"):
                    if ligne.startswith("DQB1*"):
                        ListeDQB1.append(ligne)
            if lines.startswith("DRB1"):
                for ligne in lines.split("\t"):
                    if ligne.startswith("DRB1*"):
                        ListeDRB1.append(ligne)

        body = body.append(htmltag.H2('Result for  HLA genotyping'))
        body = body.append(htmltag.P("HLA-A : "+ListeA[0]+" /  "+ListeA[1]))
        body = body.append(htmltag.P("HLA-B : "+ListeB[0]+"  / "+ListeB[1]))
        body = body.append(htmltag.P("HLA-C : "+ListeC[0]+"  / "+ListeC[1]))
        body = body.append(htmltag.P("HLA-DQB1 : "+ListeDQB1[0]+" /  "+ListeDQB1[1]))
        body = body.append(htmltag.P("HLA-DRB1 : "+ListeDRB1[0]+"  / "+ListeDRB1[1]))

        HL1A = "HLA-"+ListeA[0].split(":")[0]+":"+ListeA[0].split(":")[1]
        HL2A = "HLA-"+ListeA[1].split(":")[0]+":"+ListeA[1].split(":")[1]
        HL1B = "HLA-"+ListeB[0].split(":")[0]+":"+ListeB[0].split(":")[1]
        HL2B = "HLA-"+ListeB[1].split(":")[0]+":"+ListeB[1].split(":")[1]

        body = writeBodyHLA(HL1A,DictHLA,body)
        body = writeBodyHLA(HL2A,DictHLA,body)
        body = writeBodyHLA(HL1B,DictHLA,body)
        body = writeBodyHLA(HL2B,DictHLA,body)

    return body

def main():
    # Options
    try:
        opts, args = getopt.getopt(sys.argv[1:],"v:s:t:o:")
        for opt, arg in opts:
            if opt in ("-v"):
                vcfSNP = arg
            if opt in ("-s"):
                report = arg
            if opt in ("-t"):
                tsvfile = arg
            elif opt in ("-o"):
                outputfile = arg
    except getopt.GetoptError:
        print("please use this command as follow: ")
        print('usage : python2.7 create_html.py -v <path/to/vcf/file> -s <path/to/report/from/HLAscan> -t <path/to/pharmgkb/table> -o <path/to/file/results>')
        sys.exit(1)


    # Manage options
    if not 'vcfSNP' in vars():
        print('No vcf SNP file given')
        print('usage : python2.7 create_html.py -v <path/to/vcf/file> -s <path/to/report/from/HLAscan>  -t <path/to/pharmgkb/table> -o <path/to/file/results>')
        sys.exit(1)
    if not 'report' in vars():
        print('No report HLAscan given')
        print('usage : python2.7 create_html.py -v <path/to/vcf/file> -s <path/to/report/from/HLAscan>  -t <path/to/pharmgkb/table> -o <path/to/file/results>')
        sys.exit(1)
    if not 'tsvfile' in vars():
        print('No tsv file given')
        print('usage : python2.7 create_html.py -v <path/to/vcf/file> -s <path/to/report/from/HLAscan>  -t <path/to/pharmgkb/table> -o <path/to/file/results>')
        sys.exit(1)
    if not 'outputfile' in vars():
        print('No output file given')
        print('usage : python2.7 create_html.py -v <path/to/vcf/file> -s <path/to/report/from/HLAscan>  -t <path/to/pharmgkb/table> -o <path/to/file/results>')
        sys.exit(1)

    # Check files
    try:
        f = open(vcfSNP, 'r')
        f.close()
    except IOError:
        print('vcfSNP file does not exist')
        sys.exit(1)
    try:
        f = open(report, 'r')
        f.close()
    except IOError:
        print('Report HLAscan file does not exist')
        sys.exit(1)
    try:
        f = open(tsvfile, 'r')
        f.close()
    except IOError:
        print('PharmGKB tsv file does not exist')
        sys.exit(1)
    try:
        f = open(outputfile, 'w')
        f.close()
    except IOError:
        print("Can't open output file" )
        sys.exit(1)

    #execute
    dictRs,dictHLA = parsePharmGKB(tsvfile)

    if len(dictRs.keys()) == 0:
        print("WARNING : no rs variant found in PharmGKB file")
    if len(dictHLA.keys()) == 0:
        print("WARNING : no HLA variant found in PharmGKB file")

    head,body = contructHeader()
    body = extractRS(tsvfile,vcfSNP,dictRs,body)
    body = extractHLA(report,dictHLA,body)
    with open(outputfile,'w') as output:
        output.write(htmltag.HTML(head+body))

if __name__ == "__main__":
    main()